package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.uuid.IdGen;
import com.community.huiju.common.hk.HKOpenApi;
import com.community.huiju.dao.TaUserMapper;
import com.community.huiju.model.TaUser;
import com.community.huiju.service.IHKService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 海康 业务实现
 * @author weiximei
 */
@Service
@Slf4j
public class HKServiceImpl implements IHKService {

    @Autowired
    private TaUserMapper taUserMapper;

    private IdGen idGen = IdGen.get();

    @Override
    public ResponseBean pushPerson(String appUserId) {
        ResponseBean responseBean = new ResponseBean();
        TaUser user = taUserMapper.selectByPrimaryKey(Integer.parseInt(appUserId));
        // 这两个值一般都是同时存在的
        // 当这两个值不存在的时候，就添加海康人员
        if (null == user.getHkUserId() && null == user.getHkPersonNo()) {
            // 设置 海康id
            user.setHkPersonNo(idGen.nextId());
            /**
             * 1.获取部门(比如 住户)
             * 2.根据部门编号推送 海康
             */
            taUserMapper.updateByPrimaryKeySelective(user);
            addUserAndOpenCard(responseBean, user);
        }



        // 推送海康

        // 给用户添加 海康门禁权限
        HKOpenApi.addAuthoritiesByPersonIds(String.valueOf(user.getHkUserId()));

        // 下发门禁权限
        HKOpenApi.downloadAuthorityByDeviceUuids();

        //--------- 可视对讲 ----------
        visualIntercom(user);
        if ("0".equals(responseBean.getCode())) {
            responseBean.addSuccess("操作成功！");
            return responseBean;
        }
        responseBean.addError("操作失败!");
        return responseBean;
    }

    /**
     * 公共方法
     * 添加人员, 开卡
     *
     * @param response
     * @param user
     */
    private void addUserAndOpenCard(ResponseBean response, TaUser user) {
        Map<String, Object> parDept = Maps.newHashMap();
        parDept.put("pageNo", 1);
        parDept.put("pageSize", 100);
        parDept.put("deptName", Constant.DEPT_RESIDENTS);
        // 部门UUID
        String deptUuid = getDeptUUID(parDept);

        // 添加人员
        Map<String, Object> parUser = Maps.newHashMap();
        parUser.put("personNo", user.getHkPersonNo());
        parUser.put("personName", user.getUserName());
        parUser.put("phoneNo", user.getLoginName());
        parUser.put("deptUuid", deptUuid);
        Map<String, Object> resultMap = JSONObject.parseObject(HKOpenApi.addUser(parUser), HashMap.class);
        int errorCode = (int) resultMap.get("errorCode");
        if (errorCode == 0) {
            Map<String, Object> resultDataMap = (Map<String, Object>) resultMap.get("data");
            // 开卡 卡号
            long cardNo = System.currentTimeMillis();
            // 海康人员ID
            Integer personId = (Integer) resultDataMap.get("personId");
            TaUser tempUser = taUserMapper.selectByPrimaryKey(user.getId());
            tempUser.setHkUserId(personId);
            tempUser.setHkCardNo(cardNo + "");

            user.setHkUserId(personId);
            // 存储海康人员ID
            int row = taUserMapper.updateByPrimaryKeySelective(tempUser);
            if (row > 0) {
                response.addSuccess("操作成功!");
            } else {
                response.addError("操作失败");
                throw new RuntimeException("数据库添加 家属/租客 人员失败!");
            }

            // 开卡
            List<String> cardAndPersonList = Lists.newArrayList();
            cardAndPersonList.add(cardNo + "," + personId);
            response = openCard(cardAndPersonList, null);
        } else {
            String errorMessage = String.valueOf(resultMap.get("errorMessage"));
            response.addError(errorMessage);
            log.error("海康添加人员失败! {}", errorMessage);
            throw new RuntimeException(errorMessage);
        }
    }

    /**
     * 批量开卡
     *
     * @param cardIdAndHKUserID list.add("卡号,人员ID")
     * @param expirationTime    有效期为多少? 单位为 年
     * @return
     */
    private ResponseBean openCard(List<String> cardIdAndHKUserID, Integer expirationTime) {
        ResponseBean response = new ResponseBean();

        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        LocalDate today = LocalDate.now();
        int year = today.getYear();
        // 加上20, 表示加上20年,默认
        if (null == expirationTime || expirationTime <= 0) {
            year += 20;
        } else {
            year += expirationTime;
        }
        int month = today.getMonthValue();
        int day = today.getDayOfMonth();

        Long time = null;
        try {
            time = formatter.parse(year + "-" + month + "-" + day).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        // 开卡
        Map<String, Object> parOpenCard = Maps.newHashMap();
        parOpenCard.put("cardAndPerson", cardIdAndHKUserID);
        parOpenCard.put("startTime", System.currentTimeMillis());
        parOpenCard.put("endTime", time);
        JSONObject openCardJSON = JSONObject.parseObject(HKOpenApi.openCard(parOpenCard));
        int code = (int) openCardJSON.get("errorCode");
        if (code == 0) {
            response.addSuccess("操作成功!");
        } else {
            String errorMessage = String.valueOf(openCardJSON.get("errorMessage"));
            response.addError(errorMessage);
            log.error("开卡失败! {}", errorMessage);
            throw new RuntimeException(errorMessage);
        }

        return response;
    }

    /**
     * 获取部门 uuid
     * <p>
     * 这里的部门相当于 小区
     *
     * @return
     */
    private String getDeptUUID(Map<String, Object> parDept) {
        Map<String, Object> deptMap = JSONObject.parseObject(HKOpenApi.getDeptName(parDept), HashMap.class);
        Map<String, Object> deptDataMap = (Map<String, Object>) deptMap.get("data");
        JSONArray jsonArray = (JSONArray) deptDataMap.get("list");
        Map<String, Object> deptListData = (Map<String, Object>) jsonArray.get(0);
        // 部门UUID
        String deptUuid = (String) deptListData.get("deptUuid");
        return deptUuid;
    }

    /**
     * 可视对讲下发
     *
     * @param user
     */
    private void visualIntercom(TaUser user) {
        // TODO 目前下发的 authName权限组是我们自定义的, 后面肯定是按照小区的 期楼栋号来设置的
        // TODO longNums设备编号, 可视对讲的机器编号, 应该根据 期楼栋号 下面的机器编号来设置的

        //--------- 可视对讲 ----------

        // 可视对讲 根据人员 ID 集添加权限并下发
        Map<String, Object> addOutDoorAuthMap = new HashMap<>();
        addOutDoorAuthMap.put("authName", "荟居");
        addOutDoorAuthMap.put("personIds", Arrays.asList(user.getHkUserId()));
        addOutDoorAuthMap.put("longNums", Arrays.asList("10010100000"));
        // 可视对讲 根据人员 ID 集添加权限并下发
        HKOpenApi.addOutDoorAuth(addOutDoorAuthMap);

        // 可视对讲 指定人员指纹权限下载
//        Map<String, Object> downloadFingerMap = new HashMap<>();
//        // opType 操作类型 Integer 是 1 为下发权限；2 为删除权限
//        downloadFingerMap.put("opType", 1);
//        downloadFingerMap.put("personIds", Arrays.asList(user.getHkUserId()));
//        downloadFingerMap.put("longNums", Arrays.asList("10010100000"));
//        // 可视对讲 指定人员指纹权限下载
//        HKOpenApi.downloadFinger(downloadFingerMap);

    }
}
